from aiohttp.test_utils import AioHTTPTestCase
import peewee
import peewee_async

from core.model import Model
from core.dateutils import to_iso

import unittest
import datetime
import asyncio


class AioTestCase(unittest.TestCase):
    def setUp(self):
        self.now = to_iso(datetime.datetime.utcnow())

        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(None)

    def tearDown(self):
        self.loop.close()


class ModelTestCase(AioTestCase):
    models = []
    database = None

    def setUp(self):
        super().setUp()
        self.manager = peewee_async.Manager(self.database, loop=self.loop)
        for m in self.models:
            m._meta.database = self.database
            m.objects = self.manager
        self.database.create_tables(self.models, safe=True)

        self.create_fixtures()

    def create_fixtures(self):
        """
        method for create fixtures,
        not implemented for base test case
        :return:
        """

    def tearDown(self):
        for m in self.models:
                self.loop.run_until_complete(m.objects.close())
        self.loop.run_until_complete(self.manager.close())
        self.database.drop_tables(self.models, safe=True, cascade=True)
        super().tearDownClass()


class IntegrationTestCase(AioHTTPTestCase):
    models = []
    database = None

    def get_app(self):
        raise NotImplementedError

    def setUp(self):
        super().setUp()

        self.now = to_iso(datetime.datetime.utcnow())

        self.manager = peewee_async.Manager(self.database, loop=self.loop)
        for m in self.models:
            m._meta.database = self.database
            m.objects = self.manager
        self.database.create_tables(self.models, safe=True)

        self.create_fixtures()

    def create_fixtures(self):
        """
        method for create fixtures,
        not implemented for base test case
        :return:
        """
        pass

    def tearDown(self):
        for m in self.models:
                self.loop.run_until_complete(m.objects.close())
        self.loop.run_until_complete(self.manager.close())
        self.database.drop_tables(self.models, safe=True, cascade=True)
        super().tearDown()


class TestingPaginationModel(Model):
    date = peewee.DateTimeField()

    class Meta:
        db_table = 'test_table'



import peewee_asyncext
from app import settings


def init_db():
    return peewee_asyncext.PooledPostgresqlExtDatabase(
        settings.POSTGRES_DB,
        user=settings.POSTGRES_USER,
        password=settings.POSTGRES_PASSWORD,
        host=settings.POSTGRES_HOST,
        port=settings.POSTGRES_PORT,
        threadlocals=True,
        register_hstore=False,
    )


def init_test_db():
    return peewee_asyncext.PooledPostgresqlExtDatabase(
        settings.POSTGRES_TEST_DB,
        user=settings.POSTGRES_TEST_USER,
        password=settings.POSTGRES_TEST_PASSWORD,
        host=settings.POSTGRES_TEST_HOST,
        port=settings.POSTGRES_TEST_PORT,
        register_hstore=False,
    )


def drop_tables():
    DB.set_allow_sync(True)
    DB.drop_tables(MODELS, safe=True, cascade=True)
    print('Tables has been droped succesfully')


def create_tables(safe=False):
    DB.set_allow_sync(True)

    for m in MODELS:
        if not m._meta.db_table:
            raise AttributeError('Attribute `db_table` should be not empty')

    DB.create_tables(MODELS, safe=safe)
    print('Tables has been created succesfully')



from aiohttp import web
import jwt
from jwt.exceptions import DecodeError

from app import settings


class AnonymousUser(dict):
    def __bool__(self):
        return False

    def __str__(self):
        return 'Anonymous'


def encode(payload, secret=settings.JWT_SECRET, algorithm='HS256'):
    return jwt.encode(payload, secret, algorithm=algorithm)


def decode(encoded, secret=settings.JWT_SECRET, algorithms=['HS256']):
    return jwt.decode(encoded, secret, algorithms=algorithms)


def read_token(header):
    """
    read Authorization token from header
    :param header: Authorization token
    :return: decoded token such as a user payload or AnonymousUser
    """
    try:
        type, token = header.split(' ')
    except ValueError:
        return AnonymousUser()

    if type != 'JWT' or not token:
        return AnonymousUser()

    try:
        return decode(token)
    except DecodeError:
        return AnonymousUser()


async def auth_middleware(app, handler):
    async def middleware_handler(request):
        auth_header = request.headers.get('Authorization')
        if not auth_header:
            request.user = AnonymousUser()

        else:
            request.user = read_token(auth_header)

        return await handler(request)

    return middleware_handler


async def permissions_middleware(app, handler):
    async def middleware_handler(request):
        if not hasattr(handler, 'permissions'):
            return await handler(request)

        for perm in handler.permissions:
            response = await perm.check_permission(request)
            if response:
                return response

        return await handler(request)

    return middleware_handler


def logging_middeleware_factory(logger):
    async def logging_middleware(app, handler):
        async def middleware_handler(request):
            try:
                response = await handler(request)
                if response.status < 500:
                    return response

                logger.http_error(request, response)

                return response

            except web.HTTPException as err:
                if err.status < 500:
                    return err

                logger.http_error(request, err)
                return err

        return middleware_handler
    return logging_middleware


def content_type_setter_middleware_factory(content_type):
    async def content_type_setter_middleware(app, handler):
        async def middleware_handler(request):

            response = await handler(request)

            setattr(response, 'content_type', content_type)

            return response

        return middleware_handler
    return content_type_setter_middleware



from playhouse import shortcuts


class CoreNormalizer(object):
    fields = None

    def _model_to_dict(self, model_object):
        raise NotImplemented

    def normalize(self, queryset):
        return [self.normalize_object(i) for i in queryset]

    def normalize_object(self, model_object):
        if self.fields:
            return {field: getattr(model_object, field) for field in self.fields}
        return shortcuts.model_to_dict(model_object)


class BaseNormalizer(CoreNormalizer):
    def _model_to_dict(self, model_object):
        return shortcuts.model_to_dict(model_object)



from aiohttp import web
from cerberus import Validator

from core.json import dumps
from core.normalizers import BaseNormalizer


class BaseView(web.View):
    permissions = ()


class JsonifyResponseView(BaseView):
    def json_response(self, data, status=200):
        return web.json_response(data, dumps=dumps, status=status)


METHODS_WITH_BODY = ('POST', 'PATCH', 'PUT')


class RequestValidator(object):
    query = Validator({}, allow_unknown=True)
    headers = Validator({}, allow_unknown=True)
    params = Validator({}, allow_unknown=True)
    body = Validator({}, allow_unknown=True)

    def __init__(self, request, **kwargs):
        self.request = request
        self.query = kwargs.get('query_validator', self.query)
        self.headers = kwargs.get('header_validator', self.headers)
        self.params = kwargs.get('params_validator', self.params)
        self.body = kwargs.get('body_validator', self.body)

    async def validate(self):
        for handler in self.handlers:
            err = await handler()
            if err:
                return err

    async def validate_request(self):
        return await self.validate()

    @property
    def handlers(self):
        return (
            self.validated_query,
            self.validate_params,
            self.validated_headers,
            self.validate_body,
        )

    async def validated_query(self):
        if not self.query.validate(dict(self.request.query)):
            return self.query.errors

    async def validated_headers(self):
        if not self.headers.validate(dict(self.request.headers)):
            return self.headers.errors

    async def validate_params(self):
        if not self.params.validate(dict(self.request.match_info)):
            return self.params.errors

    async def validate_body(self):
        if self.request.method not in METHODS_WITH_BODY:
            return

        if not self.body.validate(await self.request.json()):
            return self.body.errors


class ListView(JsonifyResponseView):
    model = None
    normalizer = BaseNormalizer()
    pagination = None
    validator = RequestValidator

    def get_queryset(self):
        if not self.model:
            raise AttributeError('You should set "model" as class-view parameter')

        return self.model.select()

    def paginate(self, queryset):
        if not self.pagination:
            return queryset

        rule = self.validator.query.document.get('paginate_to', 'first')
        through = self.validator.query.document.get('paginate_through')
        last_id = self.validator.query.document.get('last_id', None)
        return getattr(self.pagination, rule)(queryset, through, last_id)

    async def normalize(self, queryset):
        executed = await self.model.objects.execute(queryset)
        return self.normalizer.normalize(executed)

    async def validate_request(self):
        validator_classes = self.init_validator_classes()
        validator = self.validator(self.request, **validator_classes)
        return await validator.validate_request()

    def init_validator_classes(self):
        return {}

    async def get(self):
        err = await self.validate_request()
        if err:
            return self.json_response(err, status=400)

        queryset = self.get_queryset()
        paginated = self.paginate(queryset)
        normalized = await self.normalize(paginated)
        return self.json_response(normalized)



import json
import datetime

from core import dateutils
from app import settings


def dumps(data):
    def handler(entity):
        if isinstance(entity, datetime.datetime):
            return dateutils.to_iso(entity)
        elif isinstance(entity, bytes):
            return entity.decode('utf-8')
        raise NotImplementedError("""
            You should implement method `dumps(self, data)`
            in your view class
            and override default json handler
        """)

    indent = None
    if settings.DEV:
        indent = 4
    return json.dumps(data, default=handler, indent=indent)



import datetime

from core.dateutils import to_iso
from models import Event, Photo, User, FavoritePhoto


photo_keys = [
    '15834214258_26b1254bff_k.jpg',
    '16019680011_1ae9064d8d_k.jpg',
    '16020945192_808c82a8a7_k.jpg',
    '21450400064_b9d7fff873_k.jpg',
    '21452101763_f0e3026be6_k.jpg',
    '21885073460_a20b85e0f1_k.jpg',
    '22047108046_4b1ee1f95d_k.jpg',
    '22061460032_ced814354b_k.jpg',
    '22073722115_dd2a142441_k.jpg',
    '22083261081_9b0546722c_k.jpg',
    'GF-runners.jpg',
    'NAF16_pack_metzger-conley-rc-juan-erin_header.jpg',
    'Runners.jpg',
    'gifts-for-runners-1101446-TwoByOne.jpg',
    'lee12-aaacooldown2.jpg',
    'runners.jpeg',
    'shutterstock_105840977.jpg',
    'shutterstock_140614036-1-800x449.jpg',
]

event_urls = [f'{i + 1}.jpg' for i in range(10)]


def log_fixtures(model):
    def factory(method):
        def decorator(*args, **kwargs):
            print(f'[INFO] "{model.__name__}" data preparing')
            method(*args, **kwargs)
            print(f'[SUCCESS] "{model.__name__}" data has been created')
        return decorator
    return factory


@log_fixtures(User)
def create_user_fixtures(number=1):
    for i in range(number):
        User.create(username=f'username{i + 1}', password=f'password{i + 1}', provider='facebook')


def create_event_past_fixtures(number=50):
    events_past = (dict(
        name=f'Name{i}',
        location=f'Location{i}',
        start_date=to_iso(datetime.datetime.utcnow() - datetime.timedelta(days=(i % 50) * 365)),
        end_date=datetime.datetime.utcnow(),
        key=event_urls[i % len(event_urls)],
        lat=40.12345612,
        lon=38.65432112,
    ) for i in range(number))
    Event.insert_many(events_past).execute()


def create_event_upcoming_fixtures(number=50):
    events_upcoming = (dict(
        name=f'Name{i}',
        location=f'Location{i}',
        start_date=to_iso(datetime.datetime.utcnow() + datetime.timedelta(days=(i % 50) * 365)),
        end_date=datetime.datetime.utcnow(),
        key=event_urls[i % len(event_urls)],
        lat=40.123456,
        lon=38.6543211,
    ) for i in range(number))
    Event.insert_many(events_upcoming).execute()


@log_fixtures(Event)
def create_event_fixtures(number=50):
    create_event_past_fixtures(number)
    create_event_upcoming_fixtures(number)


@log_fixtures(Photo)
def create_photo_fixtures(number=40, events_number=100):
    photos = [[dict(
        key=photo_keys[i % len(photo_keys)],
        bibnumbers=f'{i} {i+1} {i+2}',
        owner=1,
        event=j+1,
    ) for i in range(number)] for j in range(events_number)]
    for photo_set in photos:
        Photo.insert_many(photo_set).execute()


@log_fixtures(FavoritePhoto)
def create_favorite_photo_fixtures(number=10, users_number=1):
    favorites = [[dict(
        photo=i + 1,
        owner=j + 1,
    ) for i in range(number)] for j in range(users_number)]
    for favorite_set in favorites:
        FavoritePhoto.insert_many(favorite_set).execute()


def create_fixtures():
    create_user_fixtures()
    create_event_fixtures()
    create_photo_fixtures()
    create_favorite_photo_fixtures()



import logging
import datetime
import json


class Logger(object):
    def __init__(self, name, path, level):
        self._logger = logging.getLogger(name)

        file_handler_path = path
        file_handler = logging.FileHandler(file_handler_path)

        self._logger.addHandler(file_handler)

        self._logger.setLevel(level)

    @property
    def logger(self):
        return self._logger

    def error(self, msg):
        self.logger.error(msg)


class LoggerHTTPResponse(Logger):
    def http_error(self, request, response):
        user = 'Anonymous'
        if hasattr(request, 'user') and request.user is not None:
            user = request.user.get('id')

        msg = """
status: {status}
    timestamp: {timestamp}
    method: {method}
    path: {path}
    headers: {headers}
    user: {user}
    text: {text}
        """.format(status=response.status,
                   timestamp=str(datetime.datetime.utcnow()),
                   method=request.method,
                   path=request.path,
                   headers=json.dumps(dict(request.headers)),
                   user=user,
                   text=response.text)
        self.logger.error(msg)


class ApiDeprecatedLogger(Logger):
    def deprecated_error(self, msg):
        self.logger.error("""
error: {error}
    timestamp: {timestamp}
        """.format(error=msg, timestamp=str(datetime.datetime.utcnow())))



from setuptools import setup


with open('README.md', encoding='utf-8') as f:
    long_description=f.read()

setup(
    name='aiommy',
    version='0.0.1',

    description='A helpful tools for building web API',
    long_description=long_description,

    url='https://github.com/dennypenta/aiommy',

    author='Denis Dvornikov',
    author_email='candyboobers@gmail.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.6',
    ],

    keywords='Web API',

    packages=['aiommy'],

    install_requires=['aiohttp', 'cerberus']
)



import pytz

import datetime


FORMAT = '%Y-%m-%dT%H:%M:%SZ'


def now(timezone='UTC'):
    return pytz.timezone(timezone).localize(datetime.datetime.utcnow())


def to_iso(dt, tz_name='UTC'):
    result = dt + pytz.timezone(tz_name).utcoffset(dt)
    return result.strftime(FORMAT)


def from_iso_to_datetime(dt, tz_name='UTC'):
    return datetime.datetime.strptime(dt, FORMAT)
